==> fred-backend-zone_1.0.0.orig.tar.gz: fred-backend-zone-1.0.0/src/impl/lib/zone_data.cc <==
/*
 * Copyright (C) 2020-2021  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "src/impl/lib/zone_data.hh"
#include "src/impl/lib/exception.hh"

#include "liblog/liblog.hh"

#include <utility>

namespace Fred {
namespace Zone {
namespace Impl {
namespace Lib {

namespace {

decltype(auto) init_glue_records(const AbsoluteFqdn& domain, const AbsoluteFqdn& nameserver)
{
    using Glue = boost::optional<ZoneData::Nameserver::GlueRecords>;
    const bool has_glue = nameserver.is_subdomain_of(domain);
    if (has_glue)
    {
        return Glue{ZoneData::Nameserver::GlueRecords{}};
    }
    return Glue{boost::none};
}

}//namespace Fred::Zone::Impl::Lib::{anonymous}

ZoneData::ZoneData(ZoneData::Context ctx, Options options) noexcept
    : ctx_{std::move(ctx)},
      options_{std::move(options)}
{ }

const ZoneData::Common& ZoneData::get_common() const noexcept
{
    return ctx_.common;
}

ZoneData::Domain ZoneData::get_zone_nameservers() const
{
    return Domain{AbsoluteFqdn{Fqdn{*ctx_.common.soa_record.zone}}, ctx_.common.nameservers};
}

ZoneData::ConstIterator ZoneData::begin() const noexcept
{
    return ConstIterator{*this, 0};
}

ZoneData::ConstIterator ZoneData::end() const noexcept
{
    return ConstIterator{*this, static_cast<std::int64_t>(ctx_.domains.size())};
}

std::vector<ZoneData::DsRData> ZoneData::make_ds_records(
        const Context::KeysetList& keyset_list,
        const Context::Domain& domain_data,
        const Options::DsRecord::DigestTypes& digest_types)
{
    std::vector<DsRData> ds_records{};
    if (domain_data.keyset_id == Context::KeysetId::nullopt)
    {
        return ds_records;
    }
    auto keyset_iter = std::lower_bound(
                std::begin(keyset_list),
                std::end(keyset_list),
                *domain_data.keyset_id,
                [](const auto& keyset, const auto& keyset_id)
                {
                    return keyset.keyset_id < keyset_id;
                });
    if ((keyset_iter == std::end(keyset_list)) ||
        (keyset_iter->keyset_id != domain_data.keyset_id))
    {
        LIBLOG_ERROR("keyset {} not found", **domain_data.keyset_id);
        throw_runtime_error([]() { return "keyset not found"; });
    }
    while ((keyset_iter != std::end(keyset_list)) &&
           (keyset_iter->keyset_id == domain_data.keyset_id))
    {
        if (digest_types.empty())
        {
            throw_runtime_error([]() { return "no digest type configured"; });
        }
        std::for_each(
                std::begin(digest_types),
                std::end(digest_types),
                [&](ZoneData::DsRData::DigestType digest_type)
                {
                    ds_records.emplace_back(DsRData{domain_data.fqdn, keyset_iter->dns_key, digest_type});
                });
        ++keyset_iter;
    }
    return ds_records;
}

std::vector<ZoneData::Nameserver> ZoneData::make_nameservers(
        const AbsoluteFqdn& domain,
        const Context::Nsset::Nameservers& nameserver_list)
{
    std::vector<Nameserver> nameservers{};
    std::for_each(
            std::begin(nameserver_list),
            std::end(nameserver_list),
            [&](const auto& nameserver_data)
    {
        nameservers.emplace_back(Nameserver{domain, nameserver_data});
    });
    return nameservers;
}

std::vector<ZoneData::Nameserver> ZoneData::make_nameservers(
        const Context::NssetList& nsset_list,
        const Context::Domain& domain_data)
{
    const auto nsset_iter = std::lower_bound(
            std::begin(nsset_list),
            std::end(nsset_list),
            domain_data.nsset_id,
            [](const auto& nsset, const auto& nsset_id)
            {
                return nsset.nsset_id < nsset_id;
            });
    if ((nsset_iter == std::end(nsset_list)) ||
        (nsset_iter->nsset_id != domain_data.nsset_id))
    {
        LIBLOG_ERROR("nsset {} not found", *domain_data.nsset_id);
        throw_runtime_error([]() { return "nsset not found"; });
    }
    return make_nameservers(domain_data.fqdn, nsset_iter->nameservers);
}

ZoneData::Context::Context(const Common& common) noexcept
    : common{common}
{ }

ZoneData::Context::Domain::Domain(AbsoluteFqdn fqdn)
    : fqdn{std::move(fqdn)}
{ }

ZoneData::Nameserver::Nameserver(const AbsoluteFqdn& domain, const Context::Nsset::Nameservers::value_type& nameserver_data)
    : fqdn_{nameserver_data.first},
      glue_{init_glue_records(domain, fqdn_)}
{
    if (this->are_glue_records_permitted())
    {
        std::for_each(
                std::begin(nameserver_data.second),
                std::end(nameserver_data.second),
                [&](const auto& address)
        {
            if (address.is_v4())
            {
                glue_->a_records.emplace_back(address.to_v4());
            }
            else
            {
                glue_->aaaa_records.emplace_back(address.to_v6());
            }
        });
    }
}

bool ZoneData::Nameserver::are_glue_records_permitted() const noexcept
{
    return glue_ != boost::none;
}

const AbsoluteFqdn& ZoneData::Nameserver::get_nameserver_hostname() const noexcept
{
    return fqdn_;
}

const ZoneData::Nameserver::GlueRecords& ZoneData::Nameserver::get_glue_records() const
{
    if (!this->are_glue_records_permitted())
    {
        throw_runtime_error([]() { return "glue records are permitted just for nameserver which is subdomain of domain"; });
    }
    return *glue_;
}

ZoneData::DsRData::DsRData(
        const AbsoluteFqdn& domain_name,
        const DnsKey& dns_key,
        DigestType digest_type)
    : key_tag_{Lib::DsRecord::make_key_tag(dns_key)},
      algorithm_{dns_key.algorithm()},
      digest_type_{digest_type},
      digest_{Lib::DsRecord::make_digest(domain_name, dns_key, digest_type)}
{ }

ZoneData::DsRData::KeyTag ZoneData::DsRData::get_key_tag() const noexcept
{
    return key_tag_;
}

ZoneData::DsRData::Algorithm ZoneData::DsRData::get_algorithm() const noexcept
{
    return algorithm_;
}

ZoneData::DsRData::DigestType ZoneData::DsRData::get_digest_type() const noexcept
{
    return digest_type_;
}

const ZoneData::DsRData::Digest& ZoneData::DsRData::get_digest() const noexcept
{
    return digest_;
}

ZoneData::Domain::Domain(
        const Context& ctx,
        const Context::Domain& domain_data,
        const Options::DsRecord::DigestTypes& digest_types)
    : fqdn_{domain_data.fqdn},
      ds_r_data_{make_ds_records(ctx.keysets, domain_data, digest_types)},
      nameservers_{make_nameservers(ctx.nssets, domain_data)}
{ }

ZoneData::Domain::Domain(const AbsoluteFqdn& domain, const Context::Nsset::Nameservers& nameservers)
    : fqdn_{domain},
      ds_r_data_{},
      nameservers_{make_nameservers(domain, nameservers)}
{ }

const AbsoluteFqdn& ZoneData::Domain::get_domain_name() const noexcept
{
    return fqdn_;
}

const std::vector<ZoneData::DsRData>& ZoneData::Domain::get_ds_r_data() const noexcept
{
    return ds_r_data_;
}

const std::vector<ZoneData::Nameserver>& ZoneData::Domain::get_nameservers() const noexcept
{
    return nameservers_;
}

ZoneData::ConstIterator::ConstIterator(const ZoneData& zone_data, std::int64_t idx) noexcept
    : zone_data_{zone_data},
      idx_{idx}
{ }

ZoneData::Domain ZoneData::ConstIterator::operator*() const
{
    if ((idx_ < 0) || (zone_data_.ctx_.domains.size() <= static_cast<std::size_t>(idx_)))
    {
        throw_runtime_error([]() { return "index out of range"; });
    }
    return Domain{zone_data_.ctx_, zone_data_.ctx_.domains[idx_], zone_data_.options_.ds_record.digest_types};
}

ZoneData::ConstIterator& ZoneData::ConstIterator::operator++() noexcept
{
    ++idx_;
    return *this;
}

ZoneData::ConstIterator ZoneData::ConstIterator::operator++(int) noexcept
{
    auto copy = ConstIterator{*this};
    ++idx_;
    return copy;
}

bool operator==(const ZoneData::ConstIterator& lhs, const ZoneData::ConstIterator& rhs) noexcept
{
    return lhs.idx_ == rhs.idx_;
}

bool operator!=(const ZoneData::ConstIterator& lhs, const ZoneData::ConstIterator& rhs) noexcept
{
    return lhs.idx_ != rhs.idx_;
}

bool operator<(const ZoneData::ConstIterator& lhs, const ZoneData::ConstIterator& rhs) noexcept
{
    return lhs.idx_ < rhs.idx_;
}

std::int64_t operator-(const ZoneData::ConstIterator& lhs, const ZoneData::ConstIterator& rhs) noexcept
{
    return std::int64_t{lhs.idx_ - rhs.idx_};
}

}//namespace Fred::Zone::Impl::Lib
}//namespace Fred::Zone::Impl
}//namespace Fred::Zone
}//namespace Fred

==> fred-backend-zone_1.0.1.orig.tar.gz: fred-backend-zone-1.0.1/src/impl/lib/zone_data.cc <==
/*
 * Copyright (C) 2020-2024  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "src/impl/lib/zone_data.hh"
#include "src/impl/lib/exception.hh"

#include "liblog/liblog.hh"

#include <utility>

namespace Fred {
namespace Zone {
namespace Impl {
namespace Lib {

namespace {

decltype(auto) init_glue_records(const AbsoluteFqdn& domain, const AbsoluteFqdn& nameserver)
{
    using Glue = boost::optional<ZoneData::Nameserver::GlueRecords>;
    const bool has_glue = nameserver.is_within(domain);
    if (has_glue)
    {
        return Glue{ZoneData::Nameserver::GlueRecords{}};
    }
    return Glue{boost::none};
}

}//namespace Fred::Zone::Impl::Lib::{anonymous}

ZoneData::ZoneData(ZoneData::Context ctx, Options options) noexcept
    : ctx_{std::move(ctx)},
      options_{std::move(options)}
{ }

const ZoneData::Common& ZoneData::get_common() const noexcept
{
    return ctx_.common;
}

ZoneData::Domain ZoneData::get_zone_nameservers() const
{
    return Domain{AbsoluteFqdn{Fqdn{*ctx_.common.soa_record.zone}}, ctx_.common.nameservers};
}

ZoneData::ConstIterator ZoneData::begin() const noexcept
{
    return ConstIterator{*this, 0};
}

ZoneData::ConstIterator ZoneData::end() const noexcept
{
    return ConstIterator{*this, static_cast<std::int64_t>(ctx_.domains.size())};
}

std::vector<ZoneData::DsRData> ZoneData::make_ds_records(
        const Context::KeysetList& keyset_list,
        const Context::Domain& domain_data,
        const Options::DsRecord::DigestTypes& digest_types)
{
    std::vector<DsRData> ds_records{};
    if (domain_data.keyset_id == Context::KeysetId::nullopt)
    {
        return ds_records;
    }
    auto keyset_iter = std::lower_bound(
                std::begin(keyset_list),
                std::end(keyset_list),
                *domain_data.keyset_id,
                [](const auto& keyset, const auto& keyset_id)
                {
                    return keyset.keyset_id < keyset_id;
                });
    if ((keyset_iter == std::end(keyset_list)) ||
        (keyset_iter->keyset_id != domain_data.keyset_id))
    {
        LIBLOG_ERROR("keyset {} not found", **domain_data.keyset_id);
        throw_runtime_error([]() { return "keyset not found"; });
    }
    while ((keyset_iter != std::end(keyset_list)) &&
           (keyset_iter->keyset_id == domain_data.keyset_id))
    {
        if (digest_types.empty())
        {
            throw_runtime_error([]() { return "no digest type configured"; });
        }
        std::for_each(
                std::begin(digest_types),
                std::end(digest_types),
                [&](ZoneData::DsRData::DigestType digest_type)
                {
                    ds_records.emplace_back(DsRData{domain_data.fqdn, keyset_iter->dns_key, digest_type});
                });
        ++keyset_iter;
    }
    return ds_records;
}

std::vector<ZoneData::Nameserver> ZoneData::make_nameservers(
        const AbsoluteFqdn& domain,
        const Context::Nsset::Nameservers& nameserver_list)
{
    std::vector<Nameserver> nameservers{};
    std::for_each(
            std::begin(nameserver_list),
            std::end(nameserver_list),
            [&](const auto& nameserver_data)
    {
        nameservers.emplace_back(Nameserver{domain, nameserver_data});
    });
    return nameservers;
}

std::vector<ZoneData::Nameserver> ZoneData::make_nameservers(
        const Context::NssetList& nsset_list,
        const Context::Domain& domain_data)
{
    const auto nsset_iter = std::lower_bound(
            std::begin(nsset_list),
            std::end(nsset_list),
            domain_data.nsset_id,
            [](const auto& nsset, const auto& nsset_id)
            {
                return nsset.nsset_id < nsset_id;
            });
    if ((nsset_iter == std::end(nsset_list)) ||
        (nsset_iter->nsset_id != domain_data.nsset_id))
    {
        LIBLOG_ERROR("nsset {} not found", *domain_data.nsset_id);
        throw_runtime_error([]() { return "nsset not found"; });
    }
    return make_nameservers(domain_data.fqdn, nsset_iter->nameservers);
}

ZoneData::Context::Context(const Common& common) noexcept
    : common{common}
{ }

ZoneData::Context::Domain::Domain(AbsoluteFqdn fqdn)
    : fqdn{std::move(fqdn)}
{ }

ZoneData::Nameserver::Nameserver(const AbsoluteFqdn& domain, const Context::Nsset::Nameservers::value_type& nameserver_data)
    : fqdn_{nameserver_data.first},
      glue_{init_glue_records(domain, fqdn_)}
{
    if (this->are_glue_records_permitted())
    {
        std::for_each(
                std::begin(nameserver_data.second),
                std::end(nameserver_data.second),
                [&](const auto& address)
        {
            if (address.is_v4())
            {
                glue_->a_records.emplace_back(address.to_v4());
            }
            else
            {
                glue_->aaaa_records.emplace_back(address.to_v6());
            }
        });
    }
}

bool ZoneData::Nameserver::are_glue_records_permitted() const noexcept
{
    return glue_ != boost::none;
}

const AbsoluteFqdn& ZoneData::Nameserver::get_nameserver_hostname() const noexcept
{
    return fqdn_;
}

const ZoneData::Nameserver::GlueRecords& ZoneData::Nameserver::get_glue_records() const
{
    if (!this->are_glue_records_permitted())
    {
        throw_runtime_error([]() { return "glue records are permitted just for nameserver which is subdomain of domain"; });
    }
    return *glue_;
}

ZoneData::DsRData::DsRData(
        const AbsoluteFqdn& domain_name,
        const DnsKey& dns_key,
        DigestType digest_type)
    : key_tag_{Lib::DsRecord::make_key_tag(dns_key)},
      algorithm_{dns_key.algorithm()},
      digest_type_{digest_type},
      digest_{Lib::DsRecord::make_digest(domain_name, dns_key, digest_type)}
{ }

ZoneData::DsRData::KeyTag ZoneData::DsRData::get_key_tag() const noexcept
{
    return key_tag_;
}

ZoneData::DsRData::Algorithm ZoneData::DsRData::get_algorithm() const noexcept
{
    return algorithm_;
}

ZoneData::DsRData::DigestType ZoneData::DsRData::get_digest_type() const noexcept
{
    return digest_type_;
}

const ZoneData::DsRData::Digest& ZoneData::DsRData::get_digest() const noexcept
{
    return digest_;
}

ZoneData::Domain::Domain(
        const Context& ctx,
        const Context::Domain& domain_data,
        const Options::DsRecord::DigestTypes& digest_types)
    : fqdn_{domain_data.fqdn},
      ds_r_data_{make_ds_records(ctx.keysets, domain_data, digest_types)},
      nameservers_{make_nameservers(ctx.nssets, domain_data)}
{ }

ZoneData::Domain::Domain(const AbsoluteFqdn& domain, const Context::Nsset::Nameservers& nameservers)
    : fqdn_{domain},
      ds_r_data_{},
      nameservers_{make_nameservers(domain, nameservers)}
{ }

const AbsoluteFqdn& ZoneData::Domain::get_domain_name() const noexcept
{
    return fqdn_;
}

const std::vector<ZoneData::DsRData>& ZoneData::Domain::get_ds_r_data() const noexcept
{
    return ds_r_data_;
}

const std::vector<ZoneData::Nameserver>& ZoneData::Domain::get_nameservers() const noexcept
{
    return nameservers_;
}

ZoneData::ConstIterator::ConstIterator(const ZoneData& zone_data, std::int64_t idx) noexcept
    : zone_data_{zone_data},
      idx_{idx}
{ }

ZoneData::Domain ZoneData::ConstIterator::operator*() const
{
    if ((idx_ < 0) || (zone_data_.ctx_.domains.size() <= static_cast<std::size_t>(idx_)))
    {
        throw_runtime_error([]() { return "index out of range"; });
    }
    return Domain{zone_data_.ctx_, zone_data_.ctx_.domains[idx_], zone_data_.options_.ds_record.digest_types};
}

ZoneData::ConstIterator& ZoneData::ConstIterator::operator++() noexcept
{
    ++idx_;
    return *this;
}

ZoneData::ConstIterator ZoneData::ConstIterator::operator++(int) noexcept
{
    auto copy = ConstIterator{*this};
    ++idx_;
    return copy;
}

bool operator==(const ZoneData::ConstIterator& lhs, const ZoneData::ConstIterator& rhs) noexcept
{
    return lhs.idx_ == rhs.idx_;
}

bool operator!=(const ZoneData::ConstIterator& lhs, const ZoneData::ConstIterator& rhs) noexcept
{
    return lhs.idx_ != rhs.idx_;
}

bool operator<(const ZoneData::ConstIterator& lhs, const ZoneData::ConstIterator& rhs) noexcept
{
    return lhs.idx_ < rhs.idx_;
}

std::int64_t operator-(const ZoneData::ConstIterator& lhs, const ZoneData::ConstIterator& rhs) noexcept
{
    return std::int64_t{lhs.idx_ - rhs.idx_};
}

}//namespace Fred::Zone::Impl::Lib
}//namespace Fred::Zone::Impl
}//namespace Fred::Zone
}//namespace Fred
